--[[
Name: Stitch-1.0
Revision: $Rev: 20359 $
Author(s): Nymbia (nymbia@gmail.com)
Website: http://www.wowace.com/wiki/Stitch-1.0
Documentation: http://www.wowace.com/wiki/Stitch-1.0
SVN: http://svn.wowace.com/wowace/trunk/Stitch-1.0/Stitch-1.0/
Description: Library for tradeskill information access and queueing.
Dependencies: AceLibrary, AceEvent-2.0
]]
local MAJOR_VERSION = "Stitch-1.0"
local MINOR_VERSION = "$Rev: 20359 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceLocale-2.2") then error(MAJOR_VERSION .. " requires AceLocale-2.2") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0") end
local AceEvent = AceLibrary("AceEvent-2.0")
local ENCHANTING_STRING
local locale = GetLocale()
if locale == "enUS" then
	ENCHANTING_STRING = "Enchanting"
elseif locale == "frFR" then
	ENCHANTING_STRING = "Enchantement"
elseif locale == "deDE" then
	ENCHANTING_STRING = "Verzauberkunst"
elseif locale == "koKR" then
	ENCHANTING_STRING = "마법부여"
elseif locale == "zhCN" then
	ENCHANTING_STRING = "附魔"
elseif locale == "zhTW" then
	ENCHANTING_STRING = "é™„é­”"
end
locale = nil
local Stitch = {}
Stitch.hooks = {}
local oldDoTrade
-----------------------------------
-- Stuff for Addons to Implement --
-----------------------------------
function Stitch:EnableDataGathering(addon)
	if not tostring(addon) then error("Usage: EnableDataGathering('addon')") end
	Stitch.datagatheraddons[addon] = true
	Stitch:RegisterEvent("TRADE_SKILL_SHOW")
	Stitch:RegisterEvent("CRAFT_SHOW")
	Stitch:RegisterEvent("TRADE_SKILL_UPDATE")
	Stitch:RegisterEvent("CRAFT_UPDATE")
	if not Stitch.data then
		Stitch.data = {}
	end
end
function Stitch:DisableDataGathering(addon)
	if not addon then Stitch.data = nil Stitch.datagatheraddons = {} return end
	if not tostring(addon) then error("Usage: DisableDataGathering(['addon'])") end
	Stitch.datagatheraddons[addon] = false
	for k,v in pairs(Stitch.datagatheraddons) do
		if v then
			return
		end
	end
	Stitch:UnregisterEvent("TRADE_SKILL_SHOW")
	Stitch:UnregisterEvent("CRAFT_SHOW")
	Stitch:UnregisterEvent("TRADE_SKILL_UPDATE")
	Stitch:UnregisterEvent("CRAFT_UPDATE")
	Stitch.data = nil
end
function Stitch:EnableQueue(addon)
	if not tostring(addon) then error("Usage: EnableDataGathering('addon')") end
	Stitch.queueaddons[addon] = true
	--2.0 events
	Stitch:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "StopCastCheckUnit")
	Stitch:RegisterEvent("UNIT_SPELLCAST_FAILED", "StopCastCheckUnit")
	Stitch:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "StopCastCheckUnit")
	if not Stitch.queue then
		Stitch.queue = {}
	end
	Stitch.queueenabled = true
end
function Stitch:DisableQueue(addon)
	if not addon then Stitch.queue = nil Stitch.queueaddons = {} Stitch.queueenabled = false return end
	if not tostring(addon) then error("Usage: DisableDataGathering(['addon'])") end
	Stitch.queueaddons[addon] = false
	for k,v in pairs(Stitch.queueaddons) do
		if v then
			return
		end
	end
	Stitch:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	Stitch:UnregisterEvent("UNIT_SPELLCAST_FAILED")
	Stitch:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	Stitch.queueenabled = false
	Stitch.queue = nil
end
function Stitch:GetItemDataByIndex(profession, index)
	if (not tonumber(index)) or (not Stitch.data[profession]) then error("Usage: GetItemDataByIndex('profession', index)") end
 	if Stitch.data[profession][index] then
 		Stitch:GetNums(Stitch.data[profession][index].profession, Stitch.data[profession][index].index)
 		return Stitch.data[profession][index]
 	else
 		return nil
 	end
end
function Stitch:GetItemDataByName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	for k,v in pairs(Stitch.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) and w["name"] == name then
				Stitch:GetNums(w.profession, w.index)
				return w
			end
		end
	end
	return nil
end
function Stitch:GetItemDataByPartialName(name)
	if not tostring(name) then error("Usage: GetItemDataByPartialName('name')") end
	local result = {}
	for k,v in pairs(Stitch.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) and string.match(string.lower(w["name"]), string.lower(name)) then
				Stitch:GetNums(w.profession, w.index)
				table.insert(result,w)
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetReagentDataByIndex(profession, skillindex, reagentindex)
	if (not tonumber(skillindex)) or (not tonumber(reagentindex)) or (not Stitch.data[profession]) then error("Usage: GetItemDataByIndex('profession', index)") end
 	if Stitch.data[profession][skillindex][reagentindex] then
 		if AceLibrary:HasInstance("PeriodicTable-2.0") then
 			if AceLibrary("PeriodicTable-2.0"):ItemInSet(Stitch.data[profession][skillindex][reagentindex]["link"], "Mats - Vendor Bought") then
 				Stitch.data[profession][skillindex][reagentindex]["vendor"] = true
			end
 		end
 		Stitch.data[profession][skillindex][reagentindex]["num"] = GetItemCount(Stitch:GetIDFromLink(Stitch.data[profession][skillindex][reagentindex]["link"]))
		Stitch.data[profession][skillindex][reagentindex]["numwbank"] = GetItemCount(Stitch:GetIDFromLink(Stitch.data[profession][skillindex][reagentindex]["link"]), true)
 		return Stitch.data[profession][skillindex][reagentindex]
 	else
 		return nil
 	end
end
function Stitch:GetReagentDataByName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	local result = {}
	for k,v in pairs(Stitch.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) then
				for m,x in pairs(w) do
					if tonumber(m) and x["name"] == name then
						Stitch:GetNums(w.profession, w.index)
						table.insert(result,w)
					end
				end
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetReagentDataByPartialName(name)
	if not tostring(name) then error("Usage: GetItemDataByName('name')") end
	local result = {}
	for k,v in pairs(Stitch.data) do
		for l,w in pairs(v) do
			if (not tonumber(w)) then
				for m,x in pairs(w) do
					if tonumber(m) and string.match(string.lower(x["name"]), string.lower(name)) then
						Stitch:GetNums(w.profession, w.index)
						table.insert(result,w)
					end
				end
			end
		end
	end
	if table.getn(result) == 0 then
		return nil
	else
		return result
	end
end
function Stitch:GetQueueInfo()
	return Stitch.queue
end
function Stitch:GetQueueItemInfo(index)
	Stitch:GetNums(Stitch.data[Stitch.queue[index]["profession"]][Stitch.queue[index]["index"]].profession, Stitch.data[Stitch.queue[index]["profession"]][Stitch.queue[index]["index"]].index)
	return Stitch.data[Stitch.queue[index]["profession"]][Stitch.queue[index]["index"]]
end
function Stitch:RemoveFromQueue(index)
	table.remove(Stitch.queue, index)
	if table.getn(Stitch.queue) == 0 then
		Stitch:ClearQueue()
	end
end
function Stitch:ClearQueue()
	Stitch.queue = {}
	AceEvent:TriggerEvent("Stitch_Queue_Complete")
end
function Stitch:ProcessQueue()
	if Stitch.queue[1] and type(Stitch.queue[1]) == "table" and TradeSkillFrameTitleText and TradeSkillFrameTitleText:GetText() == Stitch.queue[1]["profession"] then
		Stitch.queuecasting = true
		return oldDoTrade(Stitch.queue[1]["index"], Stitch.queue[1]["numcasts"])
	else
		Stitch.queue = {}
		AceEvent:TriggerEvent("Stitch_Queue_Complete")
	end
end
------------
-- Events --
------------
function Stitch:TRADE_SKILL_SHOW()
	Stitch.recenttrade = TradeSkillFrameTitleText:GetText()
	Stitch:ScanTrade()
end
function Stitch:CRAFT_SHOW()
	Stitch.recentcraft = CraftFrameTitleText:GetText()
	Stitch:ScanCraft()
end
function Stitch:TRADE_SKILL_UPDATE()
	local num = GetNumTradeSkills()
	if tonumber(num) and Stitch.recenttrade and num ~= Stitch.data[Stitch.recenttrade]["num"] then
		Stitch:ScanTrade()
	end
end
function Stitch:CRAFT_UPDATE()
	local num = GetNumCrafts()
	if tonumber(num) and Stitch.recentcraft and num ~= Stitch.data[Stitch.recentcraft]["num"] then
		Stitch:ScanCraft()
	end
end
function Stitch:StopCastCheckUnit(unit)
	if unit == "player" then
		Stitch:StopCast()
	end
end
function Stitch:StopCast()
	if Stitch.queuecasting then
		if event ~= "UNIT_SPELLCAST_SUCCEEDED" then
			AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(Stitch.queue))
			Stitch.queuecasting = false
			return
		end
		Stitch.queue[1].numcasts = Stitch.queue[1].numcasts - 1
		if Stitch.queue[1].numcasts < 1 then
			Stitch:RemoveFromQueue(1)
			if table.getn(Stitch.queue) > 0 then
				AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(Stitch.queue))
			else
				AceEvent:TriggerEvent("Stitch_Queue_Complete")
			end
			Stitch.queuecasting = false
		else
			AceEvent:TriggerEvent("Stitch_Queue_Continue", table.getn(Stitch.queue))
		end
	end
end
--------------------
-- Internal Stuff --
--------------------
function Stitch.hooks:DoTradeSkill(orig, index, times)
	if Stitch.queueenabled then
		if index then Stitch:AddToQueue(index, times) end
		Stitch:ProcessQueue()
	else
		return oldDoTrade(index, times)
	end
end
function Stitch:GetNums(profession, index)
	if not profession or not index or not Stitch.data or not Stitch.data[profession] or not Stitch.data[profession][index] then return end
	Stitch.data[profession][index]["numcraftable"] = 1000
	Stitch.data[profession][index]["numcraftablewbank"] = 1000
	
	for i=1,Stitch.data[profession][index].numreagents do
		if AceLibrary:HasInstance("PeriodicTable-2.0") then
			if AceLibrary("PeriodicTable-2.0"):ItemInSet(Stitch.data[profession][index][i]["link"], "Mats - Vendor Bought") then
 				Stitch.data[profession][index][i]["vendor"] = true
			end
		end
		Stitch.data[profession][index][i]["num"] = GetItemCount(Stitch:GetIDFromLink(Stitch.data[profession][index][i]["link"]))
		Stitch.data[profession][index][i]["numwbank"] = GetItemCount(Stitch:GetIDFromLink(Stitch.data[profession][index][i]["link"]), true)
		if not Stitch.data[profession][index][i]["vendor"] then
			if Stitch.data[profession][index]["numcraftable"] > math.floor(Stitch.data[profession][index][i]["num"]/Stitch.data[profession][index][i]["needed"]) then
				Stitch.data[profession][index]["numcraftable"] = math.floor(Stitch.data[profession][index][i]["num"]/Stitch.data[profession][index][i]["needed"])
			end
			if Stitch.data[profession][index]["numcraftablewbank"] > math.floor(Stitch.data[profession][index][i]["numwbank"]/Stitch.data[profession][index][i]["needed"]) then
				Stitch.data[profession][index]["numcraftablewbank"] = math.floor(Stitch.data[profession][index][i]["numwbank"]/Stitch.data[profession][index][i]["needed"])
			end
		end
	end
	if Stitch.data[profession][index]["numcraftable"] == 1000 and Stitch.data[profession][index]["numcraftablewbank"] == 1000 then
		for i=1,Stitch.data[profession][index].numreagents do
			if Stitch.data[profession][index]["numcraftable"] > math.floor(Stitch.data[profession][index][i]["num"]/Stitch.data[profession][index][i]["needed"]) then
				Stitch.data[profession][index]["numcraftable"] = math.floor(Stitch.data[profession][index][i]["num"]/Stitch.data[profession][index][i]["needed"])
			end
			if Stitch.data[profession][index]["numcraftablewbank"] > math.floor(Stitch.data[profession][index][i]["numwbank"]/Stitch.data[profession][index][i]["needed"]) then
				Stitch.data[profession][index]["numcraftablewbank"] = math.floor(Stitch.data[profession][index][i]["numwbank"]/Stitch.data[profession][index][i]["needed"])
			end
		end
	end
end
function Stitch:GetIDFromLink(link)
	local id = string.match(link, "item:(%d+)")
	return tonumber(id)
end
function Stitch:AddToQueue(index, times)
	if Stitch.queue[1] and Stitch.queue[1]["profession"] ~= Stitch.recenttrade then
		Stitch:ClearQueue()
	end
	if not times then times = 1 end
	table.insert(Stitch.queue, {
		["profession"] = Stitch.recenttrade,
		["index"] = index,
		["numcasts"] = times,
	})
	AceEvent:TriggerEvent("Stitch_Queue_Add")
end
function Stitch:ScanCraft()
	local prof = Stitch.recentcraft
	if prof~=ENCHANTING_STRING then
		return
	end
	if not Stitch.data[prof] then
		Stitch.data[prof] = {}
	end
	Stitch.data[prof]["num"] = GetNumCrafts()
	for i=1,Stitch.data[prof]["num"] do
		local skillname, _, skilltype = GetCraftInfo(i)
		if skilltype~="header" and skillname and ((not Stitch.data[prof][i]) or (Stitch.data[prof][i]["name"] ~= skillname)) then
			Stitch.data[prof][i] = {}
			Stitch.data[prof][i]["name"] = skillname
			Stitch.data[prof][i]["index"] = i
			Stitch.data[prof][i]["profession"] = prof
			Stitch.data[prof][i]["difficulty"] = skilltype
			Stitch.data[prof][i]["nummade"] = 1
			Stitch.data[prof][i]["link"] = GetCraftItemLink(i)
			Stitch.data[prof][i]["numreagents"] = GetCraftNumReagents(i)
			local v=(GetCraftSpellFocus(i)) or " "
			if (string.len(v)>2) then
				Stitch.data[prof][i]["tools"] = BuildColoredListString(v)
			else
				Stitch.data[prof][i]["tools"] = nil
			end
			Stitch.data[prof][i]["texture"] = GetCraftIcon(i)
			if not Stitch.data[prof][i]["link"] then
				Stitch.data[prof] = {}
				return
			end
			for j=1,Stitch.data[prof][i]["numreagents"] do
				local rname, rtexture, rcount, _ = GetCraftReagentInfo(i,j)
				Stitch.data[prof][i][j] = {}
				Stitch.data[prof][i][j]["name"] = rname
				Stitch.data[prof][i][j]["link"] = GetCraftReagentItemLink(i,j)
				Stitch.data[prof][i][j]["needed"] = rcount
				Stitch.data[prof][i][j]["texture"] = rtexture
				if not Stitch.data[prof][i][j]["link"] then
					Stitch.data[prof] = {}
					return
				end
			end
		end
	end
end
function Stitch:ScanTrade()
	local prof = Stitch.recenttrade
	if not Stitch.data[prof] then
		Stitch.data[prof] = {}
	end
	Stitch.data[prof]["num"] = GetNumTradeSkills()
	for i=1,Stitch.data[prof]["num"] do
		local skillname, skilltype = GetTradeSkillInfo(i)
		if skilltype~="header" and skillname and ((not Stitch.data[prof][i]) or (Stitch.data[prof][i]["name"] ~= skillname)) then
			Stitch.data[prof][i] = {}
			Stitch.data[prof][i]["name"] = skillname
			Stitch.data[prof][i]["index"] = i
			Stitch.data[prof][i]["profession"] = prof
			Stitch.data[prof][i]["difficulty"] = skilltype
			Stitch.data[prof][i]["nummade"] = GetTradeSkillNumMade(i)
			Stitch.data[prof][i]["link"] = GetTradeSkillItemLink(i)
			Stitch.data[prof][i]["numreagents"] = GetTradeSkillNumReagents(i)
			local v=(GetTradeSkillTools(i)) or " "
			if (string.len(v)>2) then
				Stitch.data[prof][i]["tools"] = BuildColoredListString(v)
			else
				Stitch.data[prof][i]["tools"] = nil
			end
			Stitch.data[prof][i]["texture"] = GetTradeSkillIcon(i)
			if not Stitch.data[prof][i]["link"] then
				Stitch.data[prof][i] = {}
			end
			if not tonumber(Stitch.data[prof][i]["numreagents"]) then return end
			for j=1,Stitch.data[prof][i]["numreagents"] do
				local rname, rtexture, rcount, _ = GetTradeSkillReagentInfo(i,j)
				Stitch.data[prof][i][j] = {}
				Stitch.data[prof][i][j]["name"] = rname
				Stitch.data[prof][i][j]["link"] = GetTradeSkillReagentItemLink(i,j)
				Stitch.data[prof][i][j]["needed"] = rcount
				Stitch.data[prof][i][j]["texture"] = rtexture
				if not Stitch.data[prof][i][j]["link"] then
					Stitch.data[prof][i] = {}
				end
			end
		end
	end
end
----------------------
-- AceLibrary Stuff --
----------------------
local function activate(self, oldLib, oldDeactivate)
	Stitch=self
	if oldLib then
		Stitch.data = oldLib.data
		Stitch.datagatheraddons = oldLib.datagatheraddons
		Stitch.queueaddons = oldLib.queueaddons
		Stitch.queue = oldLib.queue
		Stitch.queuecasting = oldLib.queuecasting
		Stitch.hooks = oldLib.hooks
		Stitch.queueenabled = oldLib.queueenabled
	else
		oldDoTrade = DoTradeSkill
		function DoTradeSkill(index, times)
			if Stitch.hooks.DoTradeSkill then
				return Stitch.hooks.DoTradeSkill(Stitch, oldDoTrade, index, times)
			else
				return oldDoTrade(index, times)
			end
		end
	end
	if not Stitch.data then
		Stitch.data = {}
	end
	if not Stitch.queueenabled then
		Stitch.queueenabled = false
	end
	if not Stitch.queueaddons then
		Stitch.queueaddons = {}
	end
	if not Stitch.datagatheraddons then
		Stitch.datagatheraddons = {}
	end
	if not Stitch.queue then
		Stitch.queue = {}
	end
	if not Stitch.queuecasting then
		Stitch.queuecasting = false
	end
	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end
local function external(self, major, instance)
	if major == "AceEvent-2.0" then
		AceEvent = instance
		AceEvent:embed(Stitch)
		Stitch:UnregisterAllEvents()
		Stitch:CancelAllScheduledEvents()
	end
end
AceLibrary:Register(Stitch, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)




--[[
self.data = {
	professionname = {
		num = total,
		index = {
			["name"] = itemname,
			["index"] = index,
			["profession"] = profession,
			["difficulty"] = "optimal",
			["nummade"] = nummade,
			["link"] = link,
			["numreagents"] = num,
			["tools"] = "tools",
			["texture"] = "texture",
			["numcraftable"] = number,--added
			["numcraftablewbank"] = number,--added
			[reagentindex] = {
				["name"] = name,
				["link"] = link,
				["needed"] = num,
				["texture"] = texture,
				["num"] = number,--added
				["numwbank"] = number--added
			},
		}
	}
}
]]